%    $Revision: 1.0 $ $Date: 2007/04/20 10:05:31  $
%    File:
%      ganglion.m
%    Title:
%      X Ganglion Cell Model by Gaudiano
%    Author:
%      Yuki Furusawa (furusawa@nil.ist.aichi-pu.ac.jp)
%    Reference Paper:
%      Gaudiano, P. (1994)
%      Simulations of X and Y retinal ganglion cell behavior with 
%      a nonlinear push-pull model of spatiotemporal retinal processing,
%      Vision Reserch, 34, pp.1767-1784
% 

function [ydot, yinit, option] = ganglion_x(t, y, flag)

global percentCount
global Count
global frequency
global contrast
global shift
        
% Photoreceptor width, center
WIDTH    = 47;
CEN      = 24.0;

switch flag
case ''

        % state variables
        gangV      = y(1);
        photoG     = [];
        for I = 1:WIDTH,
          photoG(I)  = y(I+1);
        end

        % Shunting cell parameters
        A  = 0.0;
        B  = 8.0;
        D  = -3.0;
        M  = 20.0;
        % Receptive field parameters
        C  = 10.0;
        oC = 3.0;
        S  = 2.0;
        oS = 9.0;
        % Photoreceptor parameters
        F  = 180.0;
        G  = 20.0;
        H  = 400.0;

        % Luminance
        luminance = [];
	for I = 1:WIDTH,
	  luminance(I) = 1.0;
        end
	if t > 0.001 && t <= 0.006
	  for I = 1:WIDTH,
	    luminance(I) = 1.0+contrast*cos(2.0*pi*(shift/360.0+frequency*5.0*(I-CEN)/WIDTH));
          end
        end

        % Shunting cell
        res      = [];  % initial res
        resC     = 0.0; % initial Center receptive field
        resS     = 0.0; % initial Surround receptive field
        VC       = 0.0; % initial VC
        VS       = 0.0; % initial VS
        for I = 1:WIDTH,
	  res(I) = luminance(I)*photoG(I);
	  tmp    = -((I-CEN)*(I-CEN))/2.0;
	  resC   = resC+C*exp(tmp/(oC*oC))*res(I);
	  resS   = resS+S*exp(tmp/(oS*oS))*res(I);
	  VC     = VC+C*exp(tmp/(oC*oC));
	  VS     = VS+S*exp(tmp/(oS*oS));
        end

        % ydot
        ydot(1,:) = (-gangV*(A+M*VC+M*VS)+(B+D)*(resC-resS)+B*M*VS-D*M*VC);
        for I = 1:WIDTH,
	  ydot(I+1,:) = (F*(G-photoG(I)) - H*res(I));
        end

        % display of simulation ratio
        if (t*100/0.01) > Count
          Count = Count+1;
          fprintf('\b\b\b\b\b%3.0f%%\n', (percentCount+Count/8.0));
        end


% Initialization
case 'init'
        ydot   = 0.0:0.0001:0.01; % time span
        % initial gangV,photoG[]
        yinit(1,:)  = 5.26;
        for I = 1:WIDTH,
	 yinit(I+1,:) = 6.2068965;
        end
        option = odeset('MaxStep',0.0001);       % option
end
% Copyright 2007, FURUSAWA Yuki
% $Author: furusawa $
